'use strict';

import chai from 'chai';
import sinonChai from 'sinon-chai';
import sinon from 'sinon';
import chaiAsPromised from 'chai-as-promised';
import chaiThings from 'chai-things';
const expect = chai.expect;
chai.use(sinonChai);
chai.use(chaiAsPromised);
chai.use(chaiThings);

import nfPromiser from './nfpromiser';

describe('nfPromiser', function () {
    it('should be a function', function () {
        expect(nfPromiser).to.be.a('function');
    });
    
    it('should throw an error if the first parameter is no function', function () {
        expect(nfPromiser).to.throw(TypeError);
        expect(nfPromiser.bind(null, 10)).to.throw(TypeError);
        expect(nfPromiser.bind(null, 'test')).to.throw(TypeError);
    });
    
    describe('function invocation', function () {
        let callback;
        beforeEach(function () {
            callback = sinon.spy();
        });
        
        it('should call the given function once if nfPromiser is called once', function () {
            nfPromiser(callback);
            expect(callback).to.have.been.calledOnce;
        });
        
        it('should call the given function thrice if nfPromiser is called thrice', function () {
            nfPromiser(callback);
            nfPromiser(callback);
            nfPromiser(callback);
            expect(callback).to.have.been.calledThrice;
        });
        
        it('should call the given function with 2 as paramter if 2 is provided as second parameter', function () {
            nfPromiser(callback, 2);
            expect(callback).to.have.been.calledWith(2);
        });
        
        it('should call the given function with "test" as paramter if "test" is provided as second parameter', function () {
            nfPromiser(callback, "test");
            expect(callback).to.have.been.calledWith("test");
        });
    });
    
    describe('promises', function () {
        
        it('should return a promiser if a function is provided', function () {
            let nf = function(){};
            
            expect(nfPromiser(nf)).to.be.an.instanceof(Promise);
        });
        
        it('should resolve the promise if the function result is positive', function () {
            let nf = function (callback) {
                callback();
            };
            
            expect(nfPromiser(nf)).to.be.fulfilled;
        });
        
        it('should reject if the first parameter of the callback is an object', function () {
            let nf = function (callback) {
                callback({});
            };
            
            expect(nfPromiser(nf)).to.be.rejected;
        });
        
        it('should reject if the first parameter of the nf-callback is an error', function () {
            let nf = function (callback) {
                callback(new Error());
            }
            
            expect(nfPromiser(nf)).to.be.rejected;
        });
        
        it('should reject with "Custom Error" if the first parameter of the nf-callback is a "Custom Error" error', function () {
            let nf = function (callback) {
                callback(new Error('Custom Error'));
            }
            
            expect(nfPromiser(nf)).to.be.rejectedWith('Custom Error');
        });
        
        it('should reject if the nf-function throws an error', function () {
            let nf = function (callback) {
                throw new Error();
            }
            
            expect(nfPromiser(nf)).to.be.rejectedWith('Custom Error');
        });
        
        it('should resolve if the first parameter of the callback is null, false, "", 0, NaN or undefined', function () {
            let nf = (callback) => callback(null);
            expect(nfPromiser(nf)).to.be.fulfilled;
            
            nf = (callback) => callback('');
            expect(nfPromiser(nf)).to.be.fulfilled;

            nf = (callback) => callback(0);
            expect(nfPromiser(nf)).to.be.fulfilled;

            nf = (callback) => callback(NaN);
            expect(nfPromiser(nf)).to.be.fulfilled;

            nf = (callback) => callback(undefined);
            expect(nfPromiser(nf)).to.be.fulfilled;

            nf = (callback) => callback(false);
            expect(nfPromiser(nf)).to.be.fulfilled;
        });
        
        it('should resolve with 10 if the second parameter of the nf-callback is 10 and the first is null', function () {
            let nf = (callback) => callback(null, 10);
            expect(nfPromiser(nf)).to.eventually.equal(10);
        });
        
        it('should resolve with "test" if the second parameter of the nf-callback is "test" and the first is null', function () {
            let nf = (callback) => callback(null, 'test');
            expect(nfPromiser(nf)).to.eventually.equal('test');
        });
        
        it('should resolve if the nf-function has a timeout of 5 miliseconds', function () {
            let nf = (callback) => setTimeout(callback, 5);
            return nfPromiser(nf);
        });
        
    });
});